#include "beeper.h"

uint16_t beeper::beepTmr;
uint16_t msAccumulated;
int32_t beeper::inactivityTmr;
bool beeper::lowModelVoltage;
bool beeper::reactivate;

void beeper::initialize(void) {
  pinMode(BEEPER_PIN, OUTPUT);
  digitalWrite(BEEPER_PIN, LOW);
  beepTmr = 0;
  lowModelVoltage = false;
  activity();
  reactivate = false;
}

void beeper::beep(uint16_t ms) { // start sounding beeper for duration ms.  If beeper already active then ms is added to the existing beep time
  if (!beepTmr)
    digitalWrite(BEEPER_PIN, HIGH);
    
  beepTmr += ms;
}

uint8_t beeper::loop(uint16_t ms, uint8_t screen) {
  msAccumulated += ms;
  if (inactivityTmr < 0) {
    if (-inactivityTmr <= ms) { // start of inactivity
      if (screen != 0xFF) {
        screen = 0xFF;
      }
    }
    if (reactivate) {
      screen = 0;
      reactivate = false;
    }  
  }
  inactivityTmr += ms;
  
  if (beepTmr) {
    if (ms >= beepTmr) {
    digitalWrite(BEEPER_PIN, LOW);
    beepTmr = 0;
    } else {
      beepTmr -= ms;
    }
  } else if (inactivityTmr >=0) { // trill on inactivity
    digitalWrite(BEEPER_PIN, msAccumulated & 0x0010);
  } else if (lowModelVoltage) {
    digitalWrite(BEEPER_PIN, msAccumulated & 0x0200 ? HIGH : LOW); // half-second bleeps on low model battery voltage    
  } else {
    digitalWrite(BEEPER_PIN, LOW);
  }
  return screen;
}

void beeper::activity(void) { // reset inactivity alarm
  if (inactivityTmr >= 0) {
    digitalWrite(BEEPER_PIN, LOW);
    reactivate = true;
  }
  inactivityTmr = INACTIVITY_TIME;
}

