// config.h file for ReedyTen.  Edit the options here and recompile/upload.
#ifndef config_h
#define config_h
#include <Arduino.h>

// define only one of these supported display types - leave the others commented out. SH1106 is normal for 1.3-inch OLEDS; SSD1306 is normal for 0.96-inch OLEDS; 
// DS12864 is the 1.8-inch daylight-readable Digole DS12864LCD-4
// DS160120 is the 2-inch daylight-readable Digole DS160120LCD-51 
#define SH1106
// #define SSD1306
// #define DS12864
// #define DS160120

// The OLED displays default to a dark background and the Digole displays to a bright one.  Remove the commenting-out slashes on the following line to swap to the alternative
// #define NEGATIVE_DISPLAY

// remove comment-out slashes (//) on following line if you want PPM pulses on D0 instead of D12 - (trim button input then moves to D12)
// #define PPM_ON_D0

// comment out (with // ) the following line if your module requires high-going PPM pulses instead of low-going ones
#define PULSES_LOW

// with most RF modules it's better to pull the PPM output low and let it float high - but if your module requires the output to also be driven high, comment out (with // ) the following line and then you won't need an external pull-up resistor 
#define OPEN_OUTPUT

// positioning (and optional hiding) of control indicators on main screen, _ROW and _COL are character-sized row and column offsets from the top left corner of the display. STEER positions either or both of TOP and BOT - they can't be separated
// remove the commenting-out slashes from the appropriate HIDE_ lines if you're not using some of the controls (for example if you build a Reedy Six)
#define STEER_ROW 0
#define STEER_COL 3
#define THR_ROW 0
#define THR_COL 0
#define ELE_ROW 4
#define ELE_COL 0
#define TRIM_ROW 5
#define TRIM_COL 2
// #define HIDE_BOT
// #define HIDE_TOP
// #define HIDE_THR
// #define HIDE_ELE
// #define HIDE_TRIM

// option of throttle kill switch on A6.  Remove the comment-out slashes (//) on the #define line below to activate
// A6 is analogue input only and doesn't have an internal pull up.  To use for this function either use a SPDT switch with common to A6 and the other two terminals to VCC and GND
// or fit a pull up resistor (100k or similar) to A6 amd then only a SPST switch is needed to switch A6 down to GND
// either way, when A6 is held low it will keep the throttle output at the low position, regardless of the throttle toggle switch input.  This action operates at full speed, bypassing the "Servo Speeds" setting
// #define A6_THROTTLE_KILL

// option of timer button (or momentary switch) on A7.  Remove the comment-out slashes (//) on the #define line below to activate
// A7 is analogue input only - see A6 comments above regarding use.
// short press of at least 0.5 seconds starts/stops timer 1 and bleeps.  Keep pressed for a total of 2 seconds to reset timer 1 with a longer bleep.
// #define A7_TIMER_BUTTON

// I2C addresses for the supported display types.  OLEDS are normally 0x3C, sometimes 0x3D. Digole DS12864LCD-51 defaults to 0x27 but can be altered to other values.  Edit the one for your display type if it's different
#define SH1106_ADDR 0x3C
#define SSD1306_ADDR 0x3C
#define DS12864_ADDR 0x27
#define DS160120_ADDR 0x27

#endif
