#ifndef controls_h
#define controls_h

#include "config.h"

// ms delay before trim auto repeats
#define FIRST_TRIM_ms 500
// ms delay for trim auto repeat
#define AUTO_REPEAT_TRIM_ms 67
#define TRIM_BEEP_ms 20

template <typename T> int sgn(T val) {
  return (T(0) < val) - (val < T(0));
}

class controls {
  public:
    static void setServoSpeeds(void);
    static void start(void);
    static void scan(uint16_t mS); // check switch positions, calculate positions for controls (servoSpd). mS is milliseconds since previous call
    static bool throttleUp(void); // used by timers to check if throttle stick set higher than 25%
  private:
    static int32_t normServoSpeed, progServoSpeed; // amount to change posn values by per millisecond - calculated from normServoTime and progServoTime values from EEPROM
    static int32_t posn[5]; // positions of the virtual sticks - range +/- 2097152 (2^21)
    static int32_t tPosn[4]; // positions of trimmed virtual sticks, clipped to same +/- 2097152 range (no trim on elevator trim channel)
    static int32_t output[6]; // mixed output channels for servos.  At this stage nominal +/- 2097152 range for 100% end points, but end points can go up to 125%
    static int16_t trimTmr;
    static uint16_t noTrimTmr;
    static int32_t sweepPosn;
    static bool sweepToggle; // toggles for servo sweep direction change
    static bool trimChanged;
};

#endif
