#ifndef cppm_out_h
#define cppm_out_h
#include "config.h"

// driving 6-channel cppm signal on pin 12 (PB4) using Timer 2 interrupts - ceptimus 2015-06-25
// can optionally compile version that outputs on D0 instead of D12 (with trim button input then moved to D12) - see config.h

// The JR module pinout is:
// Top.    PPM signal to module, as described above *
//         Some internet sources say this is +6V - but not on the Turnigy.  I suspect that it may have been an RF-power-level indication signal on old non-UHF modules
//         Battery +V (11.8V on Turnigy)*
//         GND *
// Bottom. RF to Antenna (mostly hangover from non-UHF systems)

// The Futaba module pinout is:
// Nearest to corner.      PPM signal to module, as described above *
//                         Battery  +V. *
//                         RF power output signal to meter on transmitter
//                         GND *
// Furthest from corner.   RF to Antenna (mostly hangover from non-UHF systems)
//

// Only three pins marked * need be connected for operation with Arduino transmitter

class cppm_out {
  public:
    // pulse intervals in half-microsecond units - varies approx 1600~4400 with centre at ~3000 but with 600 subtracted for 300 uS low pulse
    // don't set these directly - they are public as they are accessed from the (non class member) interrupt service routine
    // but to set the 16-bit quantity in an interrupt safe way, use the set() method
    static volatile uint16_t pulseWidth[6]; // pulse widths in half-microseconds

    static void start(void); // call from setup() to initialise output pin, interrupts etc and begin outputting signal
    static void set(uint8_t n, int16_t p); // set channel[n] to position p. n is channel 0 to 5. p ~ +/- 1400
    static void set_us(uint8_t n, int16_t us); // set channel[n] pulse time to us
    static int16_t get_us(uint8_t n); // return current channel[n] pulse time in us
};
#endif
