#include "displayControls.h"
#include "display.h"
#include "eeprom.h"

uint8_t displayControls::top; // position on screen
uint8_t displayControls::left;
uint8_t displayControls::height; // in characters
uint8_t displayControls::width; // in characters
uint8_t displayControls::first; // first soft char. arranged in vertical columns from the left

int8_t displayControls::stickPosn[5];
int8_t displayControls::oldStickPosn[5];
int8_t displayControls::servoPosn[5];
int8_t displayControls::oldServoPosn[5];
int8_t displayControls::trimPosn[4];
int8_t displayControls::oldTrimPosn[4];

void displayControls::selectCanvas(uint8_t tp, uint8_t lft, uint8_t hght, uint8_t wdth, uint8_t frst) {
  top = tp;
  left = lft;
  height = hght;
  width = wdth;
  first = frst;  
}

void displayControls::makeCanvas(uint8_t tp, uint8_t lft, uint8_t hght, uint8_t wdth, uint8_t frst) {
  selectCanvas(tp, lft, hght, wdth, frst);
  for (uint8_t i = 0; i < height * width; i++) {
    _display::scClear(frst++);
  }
}

void displayControls::refreshCanvas(void) {
#ifdef HIDE_TOP
# ifdef HIDE_BOT
  if (first == STEER_FIRST) return;
# endif
#endif

#ifdef HIDE_THR
  if (first == THR_FIRST) return;
#endif

#ifdef HIDE_ELE
  if (first == ELE_FIRST) return;
#endif

#ifdef HIDE_TRIM
  if (first == TRIM_FIRST) return;
#endif

  char c[] = {(char)first, 0};
  for (uint8_t col = left; col < left + width; col++) {
    for (uint8_t row = top; row < top + height; row++) {
      _display::displayStringSmall(row, col, c, 0);
      c[0]++;
    }
  }
}

void displayControls::drawPixel(uint8_t x, uint8_t y, bool ink) { // set or reset a canvas pixel
#ifdef DS160120
  uint8_t c = first + (x / 7) * height + y / 12;
  uint8_t p = (y % 12) * 8 + x % 7;
#else
  uint8_t c = first + (x / 6) * height + y / 8;
  uint8_t p = (x % 6) * 8 + y % 8;
#endif
  if (ink) {
    _display::scSetPixel(c, p);
  } else {
    _display::scResetPixel(c, p);
  }
}

void displayControls::drawHoriz(uint8_t x1, uint8_t x2, uint8_t y, bool ink) { // set or reset a horizonal row of pixels
  for (uint8_t x = x1; x <= x2; x++) {
    drawPixel(x, y, ink);
  }
}

void displayControls::drawVert(uint8_t y1, uint8_t y2, uint8_t x, bool ink) { // set or reset a vertical column of pixels
  for (uint8_t y = y1; y <= y2; y++) {
    drawPixel(x, y, ink);
  }
}

void displayControls::drawRect(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2, bool ink) { // set or reset a rectangle (edge only)
  drawHoriz(x1, x2, y1, ink);
  drawHoriz(x1, x2, y2, ink);
  drawVert(y1+1, y2-1, x1, ink);
  drawVert(y1+1, y2-1, x2, ink);
}

void displayControls::drawStick(uint8_t x, uint8_t y, bool ink) {
  drawPixel(x+1, y, ink);
  drawPixel(x+2, y, ink);
  drawPixel(x+1, y+3, ink);
  drawPixel(x+2, y+3, ink);
  for (uint8_t i = 0; i < 4; i++) {
    drawPixel(x+i, y+1, ink);
    drawPixel(x+i, y+2, ink);
  }
}

void displayControls::drawRAservoBar(uint8_t x, uint8_t y, bool ink) {
  drawVert(y, y + 8, x, ink);
  drawVert(y, y + 8, x + 1, ink);
}

void displayControls::drawMETservoBar(uint8_t x1, uint8_t x2, uint8_t y, bool ink) {
  drawHoriz(x1, x2, y, ink);
  drawHoriz(x1, x2, y + 1, ink);
}

void displayControls::drawRAtrim(uint8_t x, uint8_t y, bool ink) {
  drawHoriz(x, x + 3, y, ink);
  drawPixel(x + 1, y - 1, ink);
  drawPixel(x + 2, y - 1, ink); 
}

void displayControls::drawMEtrim(uint8_t x, uint8_t y, bool ink) {
  drawVert(y, y + 3, x, ink);
  drawPixel(x + 1, y + 1, ink);
  drawPixel(x + 1, y + 2, ink); 
}

void displayControls::refresh(void) {
  for (uint8_t i = 0; i < 5; i++) {
    if ((stickPosn[i] != oldStickPosn[i]) || (servoPosn[i] != oldServoPosn[i]) || (i < 4 && (trimPosn[i] != oldTrimPosn[i]))) {
      if (i < 2) { // top or bot
        selectCanvas(canvasRudAil);
#if defined(HIDE_BOT) || defined(HIDE_TOP)
# ifdef HIDE_BOT
        if (i == 0) continue;
# endif
# ifdef HIDE_TOP
        if (i == 1) continue;
# endif
        // when either BOT or TOP is hidden, use the TOP position to display the remaining unhidden control indicator
        drawRAtrim(10 + oldTrimPosn[i], 9, false);
        drawRAservoBar(11 + oldServoPosn[i], 1, false);
        drawStick(10 + oldStickPosn[i] * 8, 2, false);
        drawRAtrim(10 + trimPosn[i], 9, true);
        drawRAservoBar(11 + servoPosn[i], 1, true);
        drawStick(10 + stickPosn[i] * 8, 2, true);
#else
        drawRAtrim(10 + oldTrimPosn[i], 22 - 13 * i, false);
        drawRAservoBar(11 + oldServoPosn[i], 14 - 13 * i, false);
        drawStick(10 + oldStickPosn[i] * 8, 15 - 13 * i, false);
        drawRAtrim(10 + trimPosn[i], 22 - 13 * i, true);
        drawRAservoBar(11 + servoPosn[i], 14 - 13 * i, true);
        drawStick(10 + stickPosn[i] * 8, 15 - 13 * i, true);
#endif        
      } else { // motor, elevator or trim
        if (i == 2)       selectCanvas(canvasMot);
        else if (i == 3)  selectCanvas(canvasEle);
        else              selectCanvas(canvasTrm);
        if (i < 4) drawMEtrim(2, 10 + oldTrimPosn[i], false);
        drawMETservoBar(i == 4 ? 5 : 2, 10, 11 + oldServoPosn[i], false);
        drawStick(6, 10 + oldStickPosn[i] * 8, false);
        if (i < 4) drawMEtrim(2, 10 + trimPosn[i], true);
        drawMETservoBar(i == 4 ? 5 : 2, 10, 11 + servoPosn[i], true);
        drawStick(6, 10 + stickPosn[i] * 8, true);
      }
      refreshCanvas();
      oldStickPosn[i] = stickPosn[i];
      oldServoPosn[i] = servoPosn[i];
      if (i < 4)
        oldTrimPosn[i] = trimPosn[i];
    }
  }
}

void displayControls::start(void) {
  makeCanvas(canvasRudAil);
  drawRect(0, 0, 23, 10, true);
#ifndef HIDE_BOT
# ifndef HIDE_TOP
  drawRect(0, 13, 23, 23, true); // if either of the  steering control mimics is hidden, the top section of the canvas is always used to display the remaining one
# endif
#endif
  refreshCanvas();

  makeCanvas(canvasMot);
  drawRect(1, 0, 11, 23, true);
  refreshCanvas();

  makeCanvas(canvasEle);
  drawRect(1, 0, 11, 23, true);
  refreshCanvas();

  makeCanvas(canvasTrm);
  drawRect(4, 0, 11, 23, true);
  refreshCanvas();

  for (uint8_t i = 0; i < 5; i++) {
    stickPosn[i] = 0;
    oldStickPosn[i] = 1;
    servoPosn[i] = 0;
    oldServoPosn[i] = 1;
    if (i < 4) {
      int8_t midTrim = i == 2 ? -64 : 0;
      trimPosn[i] = (eeprom::md.trimPos[i] - midTrim) / 7;
      oldTrimPosn[i] = trimPosn[i] ? 1 : -trimPosn[i];
    }
  }
  refresh();
}

