#ifndef displayControls_h
#define displayControls_h

#include "config.h"

// first soft char in each canvas (macro as these constants used in hiding detection)
#define STEER_FIRST 2
#define THR_FIRST 14
#define ELE_FIRST 20
#define TRIM_FIRST 26

// top, left, height, width, first soft char
#define canvasRudAil STEER_ROW, STEER_COL, 3, 4, STEER_FIRST
#define canvasMot THR_ROW, THR_COL, 3, 2, THR_FIRST
#define canvasEle ELE_ROW, ELE_COL, 3, 2, ELE_FIRST
#define canvasTrm TRIM_ROW, TRIM_COL, 3, 2, TRIM_FIRST
   
class displayControls {
  public:
    static void start(void); // positions soft chars on screen and intializes them
    static void refresh(void); // redraws 'stick' positions
    static int8_t stickPosn[5]; // position of each 'stick' -1, 0 or +1
    static int8_t servoPosn[5]; // position of each 'servo' indicator -8 to +8
    static int8_t trimPosn[4]; // position of each trim indicator -9 to +9

    // these next three are public because they are borrowed by the screens class for preparing the bargraph display
    static void makeCanvas(uint8_t top, uint8_t left, uint8_t height, uint8_t width, uint8_t first); // specify and clear canvas
    static void drawVert(uint8_t y1, uint8_t y2, uint8_t x, bool ink); // set or reset a vertical column of pixels
    static void drawRect(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2, bool ink); // set or reset a rectangle (edge only)  
  private:
    static int8_t oldStickPosn[5]; // previous stick positions
    static int8_t oldServoPosn[5]; // previous servo positions
    static int8_t oldTrimPosn[4]; // previous trim positions
    static uint8_t top; // position on screen
    static uint8_t left;
    static uint8_t height; // in characters
    static uint8_t width; // in characters
    static uint8_t first; // first soft char in canvas. arranged in vertical columns from the left

    static void selectCanvas(uint8_t top, uint8_t left, uint8_t height, uint8_t width, uint8_t first); // specify canvas position and size and its first soft char.  canvas is NOT cleared
    static void refreshCanvas(void); // write canvas to SSD1X06 (and therefore refresh display)   
    static void drawPixel(uint8_t x, uint8_t y, bool ink); // set a canvas pixel (if ink true) reset otherwise
    static void drawHoriz(uint8_t x1, uint8_t x2, uint8_t y, bool ink); // set or reset a horizonal row of pixels
    static void drawStick(uint8_t x, uint8_t y, bool ink); // draw a stick top 'circle' with top left at x, y.  draw when ink is true, erase when ink is false
    static void drawRAservoBar(uint8_t x, uint8_t y, bool ink); // draw the servo bar for the rudder or aileron
    static void drawMETservoBar(uint8_t x1, uint8_t x2, uint8_t y, bool ink); // draw the servo bar for the motor, elevator, or trim
    static void drawRAtrim(uint8_t x, uint8_t y, bool ink); // draw the trim indicator for the rudder or aileron    
    static void drawMEtrim(uint8_t x, uint8_t y, bool ink); // draw the trim indicator for the motor or elevator    
};

#endif

