#include "eeprom.h"
#include <EEPROM.h>

// ceptimus original for single channel 2015-09-07  Modified for Reedy Ten 2016-09-19

globalData eeprom::gd;
modelData eeprom::md;
char eeprom::modelName[14];
bool eeprom::currentModelEdited;
bool eeprom::globalDataEdited;

uint32_t eeprom::signature(void) {
  uint32_t sig = 0UL;
  for (uint16_t i = 1020; i < 1024; i++) {
    sig = (sig << 8) | EEPROM.read(i);
  }
  return sig;
}

void eeprom::writeSignature(uint32_t sig) {
  for (uint16_t i = 1023; i >= 1020; i--) {
    EEPROM.update(i, (uint8_t)(sig & 0xFFUL));
    sig >>= 8;
  }
}

void eeprom::limitModelNumber(uint8_t *modelNumber) {
  if (*modelNumber < 1) *modelNumber = 1;
  if (*modelNumber > MAX_MODEL_NO) *modelNumber = MAX_MODEL_NO;
}

void eeprom::getGD(void) {
  EEPROM.get(0, gd);
  limitModelNumber(&(gd.modelNum));
  globalDataEdited = false;
}

void eeprom::putGD(void) {
  limitModelNumber(&(gd.modelNum));
  uint8_t *p;
  p = (uint8_t *)&gd;
  for (uint16_t i = 0; i < sizeof(globalData); i++)
    EEPROM.update(i, *p++);
  globalDataEdited = false;
}

void eeprom::getMD(uint8_t modelNumber) {
  limitModelNumber(&modelNumber);
  EEPROM.get(sizeof(globalData) + (modelNumber - 1) * sizeof(modelData), md);
  currentModelEdited = false;
}

void eeprom::putMD(uint8_t modelNumber) {
  limitModelNumber(&modelNumber);
  uint16_t addr = sizeof(globalData) + (modelNumber - 1) * sizeof(modelData);
  uint8_t *p;
  p = (uint8_t *)&md;
  
  for (uint16_t i = 0; i < sizeof(modelData); i++)
    EEPROM.update(addr++, *p++);

  currentModelEdited = false;
}

char *eeprom::getModelName(uint8_t modelNumber) {
  limitModelNumber(&modelNumber);
  for (uint8_t i = 0; i < 14; i++)
    modelName[i] = EEPROM.read(sizeof(globalData) + (modelNumber - 1) * sizeof(modelData) + i);
    
  return modelName;
}

