#ifndef eeprom_h
#define eeprom_h

#include <Arduino.h>

// ceptimus original for single channel 2015-09-07  Modified for Reedy Ten 2016-09-19

struct globalData {
  uint8_t modelNum;
  uint8_t batteryRange; // theoretical value is 100 for a 10V input range (equal resistor pair into 5V range input)
  uint8_t lowBatteryAlarm; // in tenths of volts
};

struct modelData {
  char modelName[14]; // 13 chars but pad in case of \0 terminator
  uint16_t timer[2];  // each timer has 13-bit seconds (up to 136.5 mins). msb 3-bits are 'timer mode' :0 off, 1:manual down, 2:thr>10% down, 3:manual up, 4: thr>10% up, 5:manual up silent, 6:thr>10% start silent, 7:Since model chosen
  int8_t trimPos[4]; // +/- 63 steps for Rud, Ail, Ele.  -127 to 0 steps for motor.  No trim for ele trim channel
  uint8_t endPoint[12]; // these apply to the 6 output channels 0-125%
  int8_t subTrim[6]; // +/- 50% applies to to the nominal (+/- 8192) servo range, not the chosen end-point range
  uint8_t outputMix[3]; // 4 bits for each of six output channels - choices: nothing, bot, top, thr, ele, + 10 types of steer/ele mix: 50S+50E, 50S-50E, 60S+40E, 60S-40E, 40S+60E, 40S-60E, 70S+30E, 70S-30E, 30S+70E, 30S-70E; + servo sweep
  uint8_t reverse; // six bits - one for each output channel. Top bit specifies steering switch for mixing: bit set uses top switch else bottom switch
  uint8_t eleTrimRange; // for the special 'reeds' elevator trim channel as a percentage of elevator end points range 0-50%
  uint8_t trimRange; // as a percentage of end points range 0-50%
  uint8_t normServoTime; // 10mS units so 0 - 2.55 secs for travel time between specified end points
  uint8_t progServoTime; // 20mS units so 0 - 5.10 secs for progressive servos (ele trim and motor) to travel between specified end points
  uint8_t Arange; // for telemetry, in tenths of volts
  uint8_t Aalarm; // for telemetry, in tenths of volts. If set to 0 then telemetry is disabled for this model
};

#define MAX_MODEL_NO ((1020 - sizeof(globalData)) / sizeof(modelData))

class eeprom {
  private:
    static void limitModelNumber(uint8_t *modelNumber);
  public:
    static char modelName[]; // temporary copy for getModelName
    static globalData gd;
    static modelData md;
    static bool currentModelEdited;
    static bool globalDataEdited;

    static uint32_t signature(void);
    static void writeSignature(uint32_t sig);
    
    static void getGD(void);
    static void putGD(void);
    static void getMD(uint8_t modelNumber);
    static void putMD(uint8_t modelNumber);
    static char *getModelName(uint8_t modelNumber);
};
#endif

