#include "screens.h"
#include "display.h"
#include "encoder.h"
#include "cppm_out.h"
#include "eeprom.h"
#include "displayControls.h"

int16_t screens::old_us[6];
uint8_t screens::buff[22];
uint8_t screens::oldList = 0xFF;
uint8_t screens::oldItem;
uint8_t screens::listLength;
uint8_t screens::choiceMode;

uint8_t *screens::ftxt(uint8_t index, uint8_t *q) { // helper function for drawing text from flash memory table - allows included strings
  uint8_t c;
  uint8_t * p = (uint8_t *) pgm_read_word(&flashString[index]);
  for (uint8_t i = 0; i < 22; i++) {
    c = pgm_read_byte(p++);
    if (c >= 0x80) {
      if (c == 0xFF) { // choose 'top' or 'bot' for mixer source
        if (eeprom::md.reverse & 0x80) { // top
          q = ftxt(110, q);
        } else { // bot
          q = ftxt(111, q);
        }
      } else { // include other string - reduces Flash size occupied by copies of same string
        q = ftxt(c & 0x7F, q);
      }
    } else {
      *q++ = c;
      if (!c) break;
    }
  }
  return --q;
}

void screens::fTxt(uint8_t index, uint8_t posn, uint8_t rvsField) {
  ftxt(index, buff); 
  _display::displayStringSmall(posn & 07, (posn >> 3) &0x1F, (const char *)buff, rvsField);
}

uint8_t screens::fTxtLen(uint8_t index) {
  return ftxt(index, buff) - buff;
}

void screens::fixedText(uint8_t index) {
  _display::fillDisplay(' ');

  uint8_t * p = (uint8_t *) pgm_read_word(&fixed[index]);
  while ((index = pgm_read_byte(p++)) != 0xFF) {
    fTxt(index, pgm_read_byte(p++), 0);
  }
}

void screens::redrawScrolling1menu(uint8_t *p, uint8_t selected) { // p is a pointer to first of listLength items, selected item to be shown highlighted at row 4
  for (uint8_t row = 1; row <= 7; row++) {
    int8_t index = ((int8_t)selected) + row - 4;
    if (index < 0) {
      index += listLength + 1;
    } else if (index > listLength) {
      index -= listLength + 1; // item to display at this row or blank row when item == listLength
    }
    fTxt(26, row, 0); // first blank the row to erase any previous longer item
    if (index != listLength) { // except for a possible blank row between the last item in the list and the first one (when currently selected item shows the wrap)
      uint8_t item = pgm_read_byte(p + index);
      fTxt (item, (((21 - fTxtLen(item)) / 2) << 3) | row, row == 4); // draw the item centred, highlighting it when it's the currently selected one on row 4
    }
  }
}

void screens::redrawScrollType2(uint8_t selected) { // selected is model number / name to be shown highlighted at row 4
  for (uint8_t row = 1; row <= 7; row++) {
    int8_t index = ((int8_t)selected) + row - 4;
    if (index < 0) {
      index += listLength + 1;
    } else if (index > listLength) {
      index -= listLength + 1; // item to display at this row or blank row when item == listLength
    }
    fTxt(26, row, 0); // first blank the row to erase any previous longer item
    if (index != listLength) { // except for a possible blank row between the last item in the list and the first one (when currently selected item shows the wrap)
      displayModelName(index + 1, row, 2, row == 4);
    }
  }
}

void screens::redrawScrollType3(uint8_t selected) { // selected is model number / name to be shown highlighted at row 5
  for (uint8_t row = 3; row <= 7; row++) {
    int8_t index = ((int8_t)selected) + row - 5;
    if (index < 0) {
      index += listLength + 1;
    } else if (index > listLength) {
      index -= listLength + 1; // item to display at this row or blank row when item == listLength
    }
    fTxt(26, row, 0); // first blank the row to erase any previous longer item
    if (index != listLength) { // except for a possible blank row between the last item in the list and the first one (when currently selected item shows the wrap)
      displayModelName(index + 1, row, 2, row == 5);
    }
  }
}

uint8_t screens::choice(uint8_t list, uint8_t item, bool *clicked) { 
  uint8_t i;
  uint8_t *p;

  if (list == 0xFF) { // inactivity alarm screen
    oldList = 0xFF;
    oldItem = 0;
    return 0;
  }
  if (list != oldList) { // initialize display highlighting selected item, count items in list
    choiceMode = 0; // default to normal fixed position menu
    oldItem = item;
    listLength = 0;
    p = (uint8_t *) pgm_read_word(&choices[list]);
    while ((i = pgm_read_byte(p++)) != 0xFF) {
      if (!listLength && (i == 0xFE)) { // this is a scrolling-type-1 menu occupying lines 1-7 with the currently selected item on line 4
        choiceMode = 1;
        listLength = pgm_read_byte(p++); // 2nd item in list gives length for this type
        redrawScrolling1menu(p, item);
        break;
      } else if (!listLength && i == 0xFD) { // this is a scrolling-type-2 menu for selecting a model occupying lines 1-7 with the currently selected item on line 4
        choiceMode = 2;
        listLength = MAX_MODEL_NO;
        redrawScrollType2(item);
        break;
      } else if (!listLength && i == 0xFC) { // this is a scrolling-type-3 menu for selecting a model occupying lines 3-7 with the currently selected item on line 5
        choiceMode = 3;
        listLength = MAX_MODEL_NO;
        redrawScrollType3(item);
        break;
      }
      fTxt(i, pgm_read_byte(p++), item == listLength ? 1 : 0);
      listLength++;
    }
    oldList = list;
    encoder::change(0);
  } else { // subsequent scans selected item may be changed by encoder and/or calling function
    int16_t item16_t = encoder::change(item);
    while (item16_t < 0) item16_t += listLength;
    while (item16_t >= listLength) item16_t -= listLength;
    item = (uint8_t)item16_t;
  }
  
  if (item != oldItem) { 
    if (choiceMode == 1) {
      p = (uint8_t *) pgm_read_word(&choices[list]);
      p += 2; // skip past marker and listLength 
      redrawScrolling1menu(p, item);
    } else if (choiceMode == 2) {
      redrawScrollType2(item);
    }  else if (choiceMode == 3) {
      redrawScrollType3(item);
    } else { // deselect oldItem and select item using display buffer rather than items from list - allows editable items like end-point percentages to still be selected
      p = (uint8_t *) pgm_read_word(&choices[list]);
      p += oldItem << 1;
      i = pgm_read_byte_near(p++);
      uint8_t posn = pgm_read_byte_near(p);
      _display::highlightSmall(posn & 07, (posn >> 3) & 0x1F, fTxtLen(i), 0);
      p = (uint8_t *) pgm_read_word(&choices[list]);
      p += item << 1;
      i = pgm_read_byte_near(p++);
      posn = pgm_read_byte_near(p);
      _display::highlightSmall(posn & 07, (posn >> 3) & 0x1F, fTxtLen(i), 1);
    }
    oldItem = item;
  }
  *clicked = encoder::pressed;
  return item;
}

void screens::ChLabel(uint8_t column) { // Draw a column of 6 X: labels at specified column
  for (uint8_t channel = 1; channel <= 6; channel++) {
    sprintf((char *)buff, "%u:", channel);
    _display::displayStringSmall(channel, column, (const char *)buff, 0);
  }
}

void screens::displayChannels(uint8_t mode) {
  for (uint8_t channel = 0; channel < 6; channel++) {
    int16_t us = cppm_out::get_us(channel);
    if (us == old_us[channel]) continue;
    old_us[channel] = us;
#ifdef DS160120
    bargraph(((us - 1500) * 33) >> 9, channel + 1);
#else
    bargraph(((us - 1500) * 7) >> 7, channel + 1);
#endif
    if (!mode) { // percentage mode
      int16_t tenthPercent = (int16_t)((((int32_t)us - 1500) * 250) >> 7);
      sprintf((char *)buff, tenthPercent < -999 ? "---" : tenthPercent < -99 ? " --" : tenthPercent < 0 ? "  -" : "   ");
      int8_t percent = tenthPercent / 10;
      if (percent < 0) percent = -percent;
      tenthPercent %= 10;
      if (tenthPercent < 0) tenthPercent = -tenthPercent; 
      sprintf(percent > 99 ? (char *)(buff+1) : percent > 9 ? (char *)(buff+2) : (char *)(buff+3), "%u.%u", percent, tenthPercent);
      _display::displayStringSmall(channel + 1, 4, (const char *)buff, 0);
    } else { // us mode
      sprintf((char *)buff, "%4u", us);
      _display::displayStringSmall(channel + 1, 5, (const char *)buff, 0);
    }
  }
}

void screens::displayCurrentModelName(uint8_t row, uint8_t column) {
  sprintf((char *)buff, "%2u:", eeprom::gd.modelNum);
  _display::displayStringSmall(row, column, (const char *)buff, 0);  
  _display::displayStringSmall(row, column + 3, eeprom::md.modelName, 0);
}

void screens::displayModelName(uint8_t modelNum, uint8_t row, uint8_t column, uint8_t rvsField) {
  sprintf((char *)buff, "%2u:", modelNum);
  _display::displayStringSmall(row, column, (const char *)buff, rvsField);  
  _display::displayStringSmall(row, column + 3, eeprom::getModelName(modelNum), rvsField);
}

#ifdef DS160120
void screens::prepareBargraphs(void) {
  for (uint8_t i = 0; i < 6; i++)
    old_us[i] = 0;

  displayControls::makeCanvas(0, 0, 1, 25, 2); // setting up 21 soft chars: 2-26 with all required bargraph drawing characters
    
  uint8_t n = 0;
  for (uint8_t i = 1; i <= 29; i += 7) {
    displayControls::drawRect(i, 2, i + 5, 9, true); 
    for (uint8_t j = 0; j < n; j++) {
      displayControls::drawVert(3, 8, i + 4 - j, true);
    }
    n++;
  }
  n = 0;
  for (uint8_t i = 35; i <= 77; i += 7) {
    displayControls::drawRect(i, 2, i + 6, 9, true);
    displayControls::drawRect(i, 3, i + 6, 8, false);
    for (uint8_t j = 0; j < n; j++) {
      displayControls::drawVert(3, 8, i + j, true);
    }
    n++;
  }
  n = 0;
  for (uint8_t i = 84; i <= 112; i += 7) {
    displayControls::drawRect(i, 2, i + 6, 9, true);
    displayControls::drawVert(3, 8, i, false);
    displayControls::drawVert(3, 8, i + 5, true);
    for (uint8_t j = 0; j < n; j++) {
      displayControls::drawVert(3, 8, i + 4 - j, true);
    }
    n++;
  }
  n = 0;
  for (uint8_t i = 119; i <= 140; i += 7) {
    displayControls::drawRect(i, 2, i + 6, 9, true);
    for (uint8_t j = 0; j < n; j++) {
      displayControls::drawVert(3, 8, i + 1 + j, true);
    }
    n++;
  }
  displayControls::drawRect(148, 2, 154, 9, true);
  displayControls::drawRect(155, 2, 167, 9, true);
  for (uint8_t i = 156; i < 161; i++) {
    displayControls::drawVert(3, 8, i, true);
  }
  displayControls::drawRect(168, 2, 174, 9, true);
  for (uint8_t i = 169; i < 173; i++) {
    displayControls::drawVert(3, 8, i, true);
  }
}

void screens::bargraph(int8_t value, int8_t row) { // draw a horizontal bargraph with value range -33 to +33 (centre zero) at specified row occupying columns 11-20
  if (value < 0) {
    if (value < -33) value = -33;
    buff[0] = value > -29 ? 23 : -27 - value;
    buff[1] = value > -21 ? 7 : value < -26 ? 24 : value == -21 ? 25 : -8 - value;
    buff[2] = value > -14 ? 7 : value < -19 ? 24 : value == -14 ? 25 : -1 - value;
    buff[3] = value > -7 ? 7 : value < -12 ? 24 : value == -7 ? 25 : 6 - value;
    buff[4] = value < -6 ? 24 : value == -1 ? 25 : 12 - value;
    sprintf((char *)(buff + 5), "\x08\x07\x07\x07\x19");
  } else {
    if (value > 33) value = 33;
    sprintf((char *)buff, "\x17\x07\x07\x07\x07");
    buff[10] = '\0';     
    buff[9] = value < 28 ? 25 : value == 33 ? 24 : value == 32 ? 26 : value - 9;
    buff[8] = value < 21 ? 7 : value > 26 ? 24 : value - 13;
    buff[7] = value < 14 ? 7 : value > 19 ? 24 : value - 6;
    buff[6] = value < 7 ? 7 : value > 12 ? 24 : value + 1;
    buff[5] = value > 5 ? 24 : value + 8;
  }
  _display::displayStringSmall(row, 11, (const char *)buff, 0);  
}
#else
void screens::prepareBargraphs(void) {
  for (uint8_t i = 0; i < 6; i++)
    old_us[i] = 0;

  displayControls::makeCanvas(0, 0, 1, 21, 2); // setting up 21 soft chars: 2-22 with all required bargraph drawing characters
  displayControls::drawRect(1, 0, 5, 6, true);
  displayControls::drawVert(1, 5, 5, false);

    
  uint8_t n = 0;
  for (uint8_t i = 7; i <= 25; i += 6) {
    displayControls::drawRect(i, 0, i + 4, 6, true); 
    for (uint8_t j = 0; j < n; j++) {
      displayControls::drawVert(1, 5, i + 3 - j, true);
    }
    n++;
  }
  n = 0;
  for (uint8_t i = 30; i <= 66; i += 6) {
    displayControls::drawRect(i, 0, i + 5, 6, true);
    displayControls::drawRect(i, 1, i + 5, 5, false);
    for (uint8_t j = 0; j < n; j++) {
      displayControls::drawVert(1, 5, i + j, true);
    }
    n++;
  }
  n = 0;
  for (uint8_t i = 72; i <= 96; i += 6) {
    displayControls::drawRect(i, 0, i + 5, 6, true);
    displayControls::drawVert(1, 5, i, false);
    for (uint8_t j = 0; j < n; j++) {
      displayControls::drawVert(1, 5, i + 4 - j, true);
    }
    n++;
  }
  n = 0;
  for (uint8_t i = 102; i <= 120; i += 6) {
    displayControls::drawRect(i, 0, i + 5, 6, true);
    for (uint8_t j = 0; j < n; j++) {
      displayControls::drawVert(1, 5, i + 1 + j, true);
    }
    n++;
  }
}

void screens::bargraph(int8_t value, int8_t row) { // draw a horizontal bargraph with value range -28 to +28 (centre zero) at specified row occupying columns 11-20
  if (value < 0) {
    if (value < -28) value = -28;
    buff[0] = value > -25 ? 2 : -22 - value;
    buff[1] = value > -19 ? 7 : value < -23 ? 13 : -5 - value;
    buff[2] = value > -13 ? 7 : value < -17 ? 13 : 1 - value;
    buff[3] = value > -7 ? 7 : value < -11 ? 13 : 7 - value;
    buff[4] = value < -5 ? 13 : 13 - value;
    sprintf((char *)(buff + 5), "\x08\x07\x07\x07\x0E");
  } else {
    if (value > 28) value = 28;
    sprintf((char *)buff, "\x02\x07\x07\x07\x07");
    buff[10] = '\0';     
    buff[9] = value < 24 ? 14 : value == 28 ? 13 : value - 5;
    buff[8] = value < 18 ? 7 : value > 22 ? 13 : value - 10;
    buff[7] = value < 12 ? 7 : value > 16 ? 13 : value - 4;
    buff[6] = value < 6 ? 7 : value > 10 ? 13 : value + 2;
    buff[5] = value > 4 ? 13 : value + 8;
  }
  _display::displayStringSmall(row, 11, (const char *)buff, 0);  
}
#endif

void screens::decimal(uint8_t *b, uint16_t value, uint8_t len, uint8_t decPlaces)
{
  bool spaces = true;
  uint8_t exponent = decPlaces ? len - 2 : len - 1;

  for (uint8_t i = 0; i < len; i++) {
    if (decPlaces && (i == len - decPlaces - 1)) {
      b[i] = '.';
      spaces = false;
      continue;
    }
    uint16_t power = tenPower[exponent--];
    uint16_t digit = value / power;
    value -= digit * power;
    if (spaces && ((i == len - 1) || (decPlaces && (i == len - decPlaces - 2)))) {
      spaces = false;
    }
    if (!digit && spaces) {
      b[i] = ' ';
    } else {
      spaces = false;
      b[i] = digit + '0';
    }
  }
  b[len] = '\0';
}

void screens::positiveDecimal(uint16_t value, uint8_t len, uint8_t decPlaces, uint8_t row, uint8_t col, uint8_t rvsField) { // right justified with leading spaces
  decimal(buff, value, len, decPlaces);
  _display::displayStringSmall(row, col, (const char *)buff, rvsField);  
}

void screens::signedDecimal(int16_t value, uint8_t len, uint8_t decPlaces, uint8_t row, uint8_t col, uint8_t rvsField) { // '-' ' ' or '+' in leftmost positon, then right justified with leading spaces
  if (value < 0) {
    buff[0] = '-';
    value = -value;
  } else if (value > 0) {
    buff[0] = '+';
  } else {
    buff[0] = ' ';
  }
  decimal(&(buff[1]), value, (uint8_t)(len - 1), decPlaces);
  _display::displayStringSmall(row, col, (const char *)buff, rvsField);  
}

void screens::hex(uint32_t value, uint8_t len, uint8_t row, uint8_t col, uint8_t rvsField) {
  value <<= (8 - len) * 4;
  for (uint8_t i = 0; i < len; i++) {
    uint8_t c = (value & 0xF0000000UL) >> 28;
    buff[i] = c <= 9 ? c + '0' : c - 10 + 'A';
    value <<= 4;
  }
  buff[len] = '\0';
  _display::displayStringSmall(row, col, (const char *)buff, rvsField);
}

