#include "timers.h"
#include "eeprom.h"
#include "controls.h"
#include "beeper.h"
#include "display.h"
#include "screens.h"

int16_t timers::duration[2];
int16_t timers::timerSeconds[2];
bool timers::timerRunning[2];
bool timers::changed[2];
#ifdef A7_TIMER_BUTTON
uint16_t timers::A7time;
#endif

uint8_t timers::timerType[2]; // :0 off, 1:manual down, 2:thr>10% down, 3:manual up, 4: thr>10% up, 5:manual up silent, 6:thr>10% start silent, 7:Since model chosen

uint16_t timers::milliseconds;

void timers::initialize(void) {
  milliseconds = 0;
  reset(0);
  reset(1);
}

void timers::loop(uint16_t ms) {
  if (controls::throttleUp()) {
    if (timerType[0] == 6) timerRunning[0] = true;
    if (timerType[1] == 6) timerRunning[1] = true;
  }
  milliseconds += ms;
  while (milliseconds >= 1000) {
    milliseconds -= 1000;
    timerSecond(0);
    timerSecond(1);
  }
#ifdef A7_TIMER_BUTTON
  if (analogRead(A7) < 512) {
    if (A7time < 2000) {
      if ((A7time + ms >= 500) && (A7time < 500)) {
        beeper::beep(500);
        startStop(0);
      }
      A7time += ms;
      if (A7time >= 2000) {
        beeper::beep(1000);
        reset(0);
      }
    } 
  } else {
    A7time = 0;
  }
#endif
}

void timers::checkBeeper(int16_t seconds) {
  if ((seconds >= 0) && ((seconds == 30) || ((seconds <= 8) && !(seconds & 0x0001)))) { // 0, 2, 4, 6, 8, or 30 seconds remaining
    beeper::beep(750);
  }
}

void timers::timerSecond(uint8_t instance) {
  changed[instance] = true;
  if (timerRunning[instance]) {
    if ((timerType[instance] == 1) || (timerType[instance] == 2)) { // down timer
      if ((timerType[instance] == 1) || controls::throttleUp()) {
        checkBeeper(--timerSeconds[instance]);
      }
    } else { // up timer
      if ((timerType[instance] == 3) || (timerType[instance] >= 5) || controls::throttleUp()) {
        ++timerSeconds[instance];
        if ((timerType[instance] == 3) || (timerType[instance] == 4)) { // not silent type
          checkBeeper(duration[instance] - timerSeconds[instance]);
        }
      }
    }
  }
}

void timers::startStop(uint8_t instance) { // 'S' from main page 
  if ((timerType[instance] == 1) || (timerType[instance] == 3) || (timerType[instance] == 5)) { // timer can be started and stopped
    timerRunning[instance] = !timerRunning[instance];
  } else if (timerType[instance] == 6) { // timer type 6 can be manually stopped but restarts on throttle
    timerRunning[instance] = false;
  } else if (timerType[instance] != 7) { // for timers that can't be manually started or stopped 'S' does the same job as 'R'
    reset(instance);
  }
}

void timers::reset(uint8_t instance) {
  timerType[instance] = (uint8_t)((eeprom::md.timer[instance] >> 13) & 0x0007);
  duration[instance] = (int16_t)(eeprom::md.timer[instance] & 0x1FFF);
  changed[instance] = true;

  if ((timerType[instance] == 1) || (timerType[instance] == 2)) { // down timer
    timerSeconds[instance] = duration[instance];
  } else { // off or up timer
    timerSeconds[instance] = 0;
  }
  timerRunning[instance] = (timerType[instance] == 2) || (timerType[instance] == 4) || (timerType[instance] == 7); // running (though maybe throttle controlled)
}

void timers::show(uint8_t instance, uint8_t row, uint8_t column, uint8_t rvsField) {
  if (!changed[instance]) return;
  changed[instance] = false;
  if (!timerType[instance]) {
    _display::displayStringSmall(row, column, F("      "), 0);
  } else {
    displaySeconds(timerSeconds[instance], row, column, rvsField);
  }
}

void timers::showDuration(uint8_t instance, uint8_t row, uint8_t column, uint8_t rvsField) {
    displaySeconds(duration[instance], row, column, rvsField);
}

void timers::displaySeconds(int16_t seconds, uint8_t row, uint8_t column, uint8_t rvsField) {
  int i;
  char buff[7];

  if (seconds < 0) {
    if (seconds < -5999) seconds = -5999;
    buff[0] = '-';
    seconds = -seconds;
  } else {
    if (seconds > 5999) seconds = 5999;
    buff[0] = ' ';
  }
  int16_t minutes = seconds / 60; seconds -= minutes * 60;
  i = minutes / 10; minutes -= i * 10;
  buff[1] = i > 0 ? i + '0' : ' ';
  buff[2] = minutes % 10 + '0';
  buff[3] = ':';
  i = seconds / 10; seconds -= i * 10;
  buff[4] = i + '0';
  buff[5] = seconds + '0';
  buff[6] = '\0';
  _display::displayStringSmall(row, column, buff, rvsField);
}

void timers::showType(uint8_t instance, uint8_t posn, uint8_t rvsField) {
  screens::fTxt(timerType[instance] + 51, posn, rvsField);
}


